@@ -149,7 +149,12 @@ def linear_beta_schedule(timesteps):
     return torch.linspace(beta_start, beta_end, timesteps, dtype = torch.float64)
 
 class GaussianDiffusion(nn.Module):
-    def __init__(self, *, beta_schedule, timesteps):
+    def __init__(
+        self,
+        *,
+        beta_schedule,
+        timesteps
+    ):
         super().__init__()
 
         if beta_schedule == "cosine":
@@ -222,13 +227,18 @@ class GaussianDiffusion(nn.Module):
         posterior_log_variance_clipped = extract(self.posterior_log_variance_clipped, t, x_t.shape)
         return posterior_mean, posterior_variance, posterior_log_variance_clipped
 
-    def q_sample(self, x_start, t, noise=None):
+    def q_sample(self, x_start, t, noise = None):
         noise = default(noise, lambda: torch.randn_like(x_start))
 
-        return (
+        noised = (
             extract(self.sqrt_alphas_cumprod, t, x_start.shape) * x_start +
             extract(self.sqrt_one_minus_alphas_cumprod, t, x_start.shape) * noise
-        ), None
+        )
+
+        alphas_cumprod = extract(self.alphas_cumprod, t, t.shape)
+        log_snr = -log(1. / alphas_cumprod - 1)
+
+        return noised, log_snr
 
     def predict_start_from_noise(self, x_t, t, noise):
         return (
@@ -236,7 +246,7 @@ class GaussianDiffusion(nn.Module):
             extract(self.sqrt_recipm1_alphas_cumprod, t, x_t.shape) * noise
         )
 
-# gaussian diffusion with continuos time helper functions and classes
+# gaussian diffusion with continuous time helper functions and classes
 # large part of this was thanks to @crowsonkb at https://github.com/crowsonkb/v-diffusion-jax/blob/master/diffusion/utils.py
 
 @torch.jit.script
@@ -968,6 +978,11 @@ class Unet(nn.Module):
             Rearrange('b (r d) -> b r d', r = num_time_tokens)
         )
 
+        self.to_lowres_time_tokens = nn.Sequential(
+            nn.Linear(time_cond_dim, cond_dim * num_time_tokens),
+            Rearrange('b (r d) -> b r d', r = num_time_tokens)
+        ) if lowres_cond else None
+
         self.to_time_cond = nn.Sequential(
             nn.Linear(time_cond_dim, time_cond_dim)
         )
@@ -1163,6 +1178,9 @@ class Unet(nn.Module):
         # derive time tokens
 
         time_tokens = self.to_time_tokens(time_hiddens)
+
+        # get time hiddens
+
         t = self.to_time_cond(time_hiddens)
 
         # text conditioning
@@ -1324,6 +1342,7 @@ class Imagen(nn.Module):
         loss_type = 'l2',
         beta_schedules = 'cosine',
         lowres_sample_noise_level = 0.2,            # in the paper, they present a new trick where they noise the lowres conditioning image, and at sample time, fix it to a certain level (0.1 or 0.3) - the unets are also made to be conditioned on this noise level
+        per_sample_random_aug_noise_level = False,  # unclear when conditioning on augmentation noise level, whether each batch element receives a random aug noise value - turning off due to @marunine's find
         condition_on_text = True,
         auto_normalize_img = True,                  # whether to take care of normalizing the image from [0, 1] to [-1, 1] and back automatically - you can turn this off if you want to pass in the [-1, 1] ranged image yourself from the dataloader
         continuous_times = True,
@@ -1410,6 +1429,7 @@ class Imagen(nn.Module):
         assert lowres_conditions == (False, *((True,) * (num_unets - 1))), 'the first unet must be unconditioned (by low resolution image), and the rest of the unets must have `lowres_cond` set to True'
 
         self.lowres_sample_noise_level = lowres_sample_noise_level
+        self.per_sample_random_aug_noise_level = per_sample_random_aug_noise_level
 
         # classifier free guidance
 
@@ -1616,7 +1636,7 @@ class Imagen(nn.Module):
 
         # get x_t
 
-        x_noisy, maybe_log_snr = noise_scheduler.q_sample(x_start = x_start, t = times, noise = noise)
+        x_noisy, log_snr = noise_scheduler.q_sample(x_start = x_start, t = times, noise = noise)
 
         # also noise the lowres conditioning image
         # at sample time, they then fix the noise level of 0.1 - 0.3
@@ -1641,8 +1661,8 @@ class Imagen(nn.Module):
         losses = self.loss_fn(pred, noise, reduction = 'none')
         losses = reduce(losses, 'b ... -> b', 'mean')
 
-        if isinstance(noise_scheduler, GaussianDiffusionContinuousTimes) and self.p2_loss_weight_gamma > 0:
-            loss_weight = (self.p2_loss_weight_k + maybe_log_snr.exp()) ** -self.p2_loss_weight_gamma
+        if self.p2_loss_weight_gamma > 0:
+            loss_weight = (self.p2_loss_weight_k + log_snr.exp()) ** -self.p2_loss_weight_gamma
             losses = losses * loss_weight
 
         return losses.mean()
@@ -1686,7 +1706,12 @@ class Imagen(nn.Module):
         if exists(prev_image_size):
             lowres_cond_img = resize_image_to(images, prev_image_size)
             lowres_cond_img = resize_image_to(lowres_cond_img, target_image_size)
-            lowres_aug_times = noise_scheduler.sample_random_times(b, device = device)
+
+            if self.per_sample_random_aug_noise_level:
+                lowres_aug_times = noise_scheduler.sample_random_times(b, device = device)
+            else:
+                lowres_aug_time = noise_scheduler.sample_random_times(1, device = device)
+                lowres_aug_times = repeat(lowres_aug_time, '1 -> b', b = b)
 
         images = resize_image_to(images, target_image_size)
 
